function [b,rts,ia,nexact,nnumeric,lgroots,aimcode] = ...
                        pcaim_eig(h,neq,nlag,nlead,condn,uprbnd)

%  [b,rts,ia,nexact,nnumeric,lgroots,aimcode] = ...
%                       aim_eig(h,neq,nlag,nlead,condn,uprbnd)
%
%  Solve a linear perfect foresight model using the matlab eig
%  function to find the invariant subspace associated with the big
%  roots.  This procedure will fail if the companion matrix is
%  defective and does not have a linearly independent set of
%  eigenvectors associated with the big roots.
% 
%  Input arguments:
% 
%    h         Structural coefficient matrix (neq,neq*(nlag+1+nlead)).
%    neq       Number of equations.
%    nlag      Number of lags.
%    nlead     Number of leads.
%    condn     Zero tolerance used as a condition number test
%              by numeric_shift and reduced_form.
%    uprbnd    Inclusive upper bound for the modulus of roots
%              allowed in the reduced form.
% 
%  Output arguments:
% 
%    b         Reduced form coefficient matrix (neq,neq*nlag).
%    rts       Roots returned by eig.
%    ia        Dimension of companion matrix (number of non-trivial
%              elements in rts).
%    nexact    Number of exact shiftrights.
%    nnumeric  Number of numeric shiftrights.
%    lgroots   Number of roots greater in modulus than uprbnd.
%    aimcode     Return code: see function aimerr.

warning off

if(nlag<1 | nlead<1) 
    error('Aim_eig: model must have at least one lag and one lead.');
end
%myt1=cputime;

% Initialization.

nexact   = 0;
nnumeric = 0;
lgroots  = 0;
iq       = 0;
aimcode    = 0;

qrows = neq*nlead;
qcols = neq*(nlag+nlead);
bcols = neq*nlag;
 
q        = zeros(qrows,qcols);
rts      = zeros(qcols,1);

% Compute the auxiliary initial conditions and store them in q.

[h,q,iq,nexact] = pcexact_shift(h,q,iq,qrows,qcols,neq);
   if (iq>qrows) 
      aimcode = 61;
      return;
   end
%disp(['Elapsed time for exact shifts    = ',num2str(cputime-myt1)]);
%myt1=cputime;

[h,q,iq,nnumeric] = pcnumeric_shift(h,q,iq,qrows,qcols,neq,condn);
   if (iq>qrows) 
      aimcode = 62;
      return;
   end

%disp(['Elapsed time for numeric shifts    = ',num2str(cputime-myt1)]);
%myt1=cputime;

%  Build the companion matrix.  Compute the stability conditions, and
%  combine them with the auxiliary initial conditions in q.  

[a,ia,js] = pcbuild_a(h,qcols,neq);

%disp(['Elapsed time for build    = ',num2str(cputime-myt1)]);
%myt1=cputime;

if (ia ~= 0)

   [w,rts,lgroots] = pceigensystem(a,uprbnd);
   if ~(all(finite(w(:))) & all(finite(rts(:))) & all(finite(lgroots(:)))),
     aimcode = 51;
     return;
   end;

   q = pccopy_w(q,w,js,iq,qrows);

end


   test = nexact+nnumeric+lgroots;
       if (test > qrows) aimcode = 3;
   elseif (test < qrows) aimcode = 4;
   end

%disp(['Elapsed time for eigensyetm    = ',num2str(cputime-myt1)]);
%myt1=cputime;
 
% If the right-hand block of q is invertible, compute the reduced form.

[nonsing,b] = pcreduced_form(q,qrows,qcols,bcols,neq,condn);

    if ( nonsing & aimcode==0) aimcode =  1;
elseif (~nonsing & aimcode==0) aimcode =  5;
elseif (~nonsing & aimcode==3) aimcode = 35;
elseif (~nonsing & aimcode==4) aimcode = 45;
end

%disp(['Elapsed time for reduced form    = ',num2str(cputime-myt1)]);

warning backtrace

return

